package org.ow2.petals.binding.soap;

/**
 * @author chamerling
 * 
 */
public final class Constants {

	/**
	 * Axis2 related constants
	 * 
	 * @author Christophe HAMERLING - eBM WebSourcing
	 * 
	 */
	public static class Axis2 {

		public static final String AXIS2_XML = "axis2.xml";

		public static final String SERVICES_XML = "services.xml";

		public static final String MODULE_ARCHIVE_EXTENSION = "mar";

		public static final String MODULES_PATH = "modules";

		public static final String SERVICES_PATH = "services";
		
		public static final String ADDRESSING_MODULE = "addressing";

	}

	/**
	 * JBI message related constants
	 * 
	 * @author Christophe HAMERLING - eBM WebSourcing
	 * 
	 */
	public static class JbiMessage {
		private static final String WS_PREFIX = "org.ow2.petals.ws";

		public static final String SOAP_SECURITY = WS_PREFIX + ".security";

		public static final String SOAP_VERSION = WS_PREFIX + ".version";

		public static final String SOAP_HEADER = WS_PREFIX + ".header";
		
		public static final String SOAP_USERNAME = WS_PREFIX + ".username";
		
		public static final String SOAP_PASSWORD = WS_PREFIX + ".password";
	}

	/**
	 * Service Unit related constants
	 * 
	 * @author Christophe HAMERLING - eBM WebSourcing
	 * 
	 */
	public static class ServiceUnit {
		/**
		 * Delete of not the root element of the soap message on incoming calls.
		 * Add or not a root element on outgoing soap message.
		 */
		public static final String ROOT_ELEMENT = "root-element";

		public static final String SOAP_ACTION = "soap-action";

		public static final String ADDRESS = "address";

		/**
		 * The modules to be used as CSV. The Addressing module is always
		 * included by default.
		 */
		public static final String MODULES = "modules";

		/**
		 * The service parameters, used to configure service.
		 */
		public static final String SERVICE_PARAMETERS = "service-parameters";

		/**
		 * The soap version of the outgoing message. Default is 1.1.
		 */
		public static final String SOAP_VERSION = "soap-version";
		
		public static final String CLEANUP_TRANSPORT = "cleanup-transport";
		
		public static final String CHUNKED_MODE = "chunked-mode";

		public static final String PROXY_HOST = "proxy-host";

		public static final String PROXY_PORT = "proxy-port";

		public static final String PROXY_USER = "proxy-user";

		public static final String PROXY_PASSWORD = "proxy-password";

		public static final String PROXY_DOMAIN = "proxy-domain";

		public static final String TOPIC_PREFIX = "topic";

		/**
		 * Basic authentication username
		 */
		public static final String BASIC_AUTH_USERNAME = "authentication.basic.username";

		/**
		 * Basic authentication password
		 */
		public static final String BASIC_AUTH_PASSWORD = "authentication.basic.password";

		/**
		 * Add a root element on outgoing soap messages.
		 */
		public static final String ADD_ROOT = "add-root";

		/**
		 * Remove the root element from incoming soap messages before building
		 * the normalized message.
		 */
		public static final String REMOVE_ROOT = "remove-root";
	}

	/**
	 * HTTP server related constants
	 * 
	 * @author Christophe HAMERLING - eBM WebSourcing
	 * 
	 */
	public static class HttpServer {
		/**
		 * The HTTP port the Jetty server listen incoming request on
		 */
		public static final String HTTP_PORT = "http.port";

		/**
		 * Provide or not services list
		 */
		public static final String HTTP_SERVICES_LIST = "http.services.list";

		/**
		 * The URL context
		 */
		public static final String HTTP_SERVICES_CONTEXT = "http.services.context";

		/**
		 * The service mapping URL
		 */
		public static final String HTTP_SERVICES_MAPPING = "http.services.mapping";

		/**
		 * The max size of the HTTP pool
		 */
		public static final String HTTP_THREAD_POOL_SIZE_MAX = "http.thread.pool.size.max";

		/**
		 * The min size of the HTTP pool
		 */
		public static final String HTTP_THREAD_POOL_SIZE_MIN = "http.thread.pool.size.min";

		/**
		 * The number of HTTP acceptors
		 */
		public static final String HTTP_ACCEPTORS = "http.acceptors";

		/* Default values */

		public static final int DEFAULT_HTTP_PORT = 8084;

		public static final boolean DEFAULT_HTTP_SERVICES_LIST = true;

		public static final String DEFAULT_PROTOCOL = "http";

		public static final String DEFAULT_HTTP_SERVICES_CONTEXT = "petals";

		public static final String DEFAULT_HTTP_SERVICES_MAPPING = "services";

		public static final int DEFAULT_HTTP_THREAD_POOL_SIZE_MAX = 255;

		public static final int DEFAULT_HTTP_THREAD_POOL_SIZE_MIN = 1;

		public static final int DEFAULT_HTTP_ACCEPTORS = 4;
	}
}
