/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap;

import static org.ow2.petals.binding.soap.Constants.HttpServer.DEFAULT_HTTP_ACCEPTORS;
import static org.ow2.petals.binding.soap.Constants.HttpServer.DEFAULT_HTTP_PORT;
import static org.ow2.petals.binding.soap.Constants.HttpServer.DEFAULT_HTTP_SERVICES_CONTEXT;
import static org.ow2.petals.binding.soap.Constants.HttpServer.DEFAULT_HTTP_SERVICES_LIST;
import static org.ow2.petals.binding.soap.Constants.HttpServer.DEFAULT_HTTP_SERVICES_MAPPING;
import static org.ow2.petals.binding.soap.Constants.HttpServer.DEFAULT_HTTP_THREAD_POOL_SIZE_MAX;
import static org.ow2.petals.binding.soap.Constants.HttpServer.DEFAULT_HTTP_THREAD_POOL_SIZE_MIN;
import static org.ow2.petals.binding.soap.Constants.HttpServer.DEFAULT_PROTOCOL;
import static org.ow2.petals.binding.soap.Constants.HttpServer.HTTP_ACCEPTORS;
import static org.ow2.petals.binding.soap.Constants.HttpServer.HTTP_PORT;
import static org.ow2.petals.binding.soap.Constants.HttpServer.HTTP_SERVICES_CONTEXT;
import static org.ow2.petals.binding.soap.Constants.HttpServer.HTTP_SERVICES_LIST;
import static org.ow2.petals.binding.soap.Constants.HttpServer.HTTP_SERVICES_MAPPING;
import static org.ow2.petals.binding.soap.Constants.HttpServer.HTTP_THREAD_POOL_SIZE_MAX;
import static org.ow2.petals.binding.soap.Constants.HttpServer.HTTP_THREAD_POOL_SIZE_MIN;

import java.util.HashMap;
import java.util.Map;

import org.ow2.petals.component.framework.ComponentConfiguration;

/**
 * The SOAP component properties
 *
 * @author Christophe HAMERLING - eBMWebSourcing
 *
 */
public class SoapProperties {

    private ComponentConfiguration componentConfiguration;

    /**
     * Creates a new instance of {@link SoapProperties}
     *
     * @param properties
     */
    public SoapProperties(ComponentConfiguration componentConfiguration) {
        this.componentConfiguration = componentConfiguration;
    }

    /**
     * Get the HTTP port
     *
     * @return
     */
    public int getHttpPort() {
        return Integer.parseInt(componentConfiguration.getProperties()
                .getProperty(HTTP_PORT, Integer.toString(DEFAULT_HTTP_PORT)));
    }

    /**
     * Does the component have to provide the services list
     *
     * @return
     */
    public boolean isProvidingServicesList() {
        return Boolean.parseBoolean(componentConfiguration.getProperties()
                .getProperty(HTTP_SERVICES_LIST,
                        Boolean.toString(DEFAULT_HTTP_SERVICES_LIST)));
    }

    public String getServicesContext() {
        return componentConfiguration.getProperties().getProperty(
                HTTP_SERVICES_CONTEXT, DEFAULT_HTTP_SERVICES_CONTEXT);
    }

    public String getServicesMapping() {
        return componentConfiguration.getProperties().getProperty(
                HTTP_SERVICES_MAPPING, DEFAULT_HTTP_SERVICES_MAPPING);
    }

    /**
     * Currently support only http protocol
     *
     * @return
     */
    public String getProtocol() {
        return DEFAULT_PROTOCOL;
    }

    /**
     * Get the thread max pool size of the Jetty server
     *
     * @return
     */
    public int getHttpThreadMaxPoolSize() {
        return Integer.parseInt(componentConfiguration.getProperties()
                .getProperty(HTTP_THREAD_POOL_SIZE_MAX,
                        Integer.toString(DEFAULT_HTTP_THREAD_POOL_SIZE_MAX)));
    }

    /**
     * Get the thread min pool size of the Jetty server
     *
     * @return
     */
    public int getHttpThreadMinPoolSize() {
        return Integer.parseInt(componentConfiguration.getProperties()
                .getProperty(HTTP_THREAD_POOL_SIZE_MIN,
                        Integer.toString(DEFAULT_HTTP_THREAD_POOL_SIZE_MIN)));
    }

    /**
     * Get the number of acceptors of the Jetty server
     *
     * @return
     */
    public int getHttpAcceptors() {
        return Integer.parseInt(componentConfiguration.getProperties()
                .getProperty(HTTP_ACCEPTORS,
                        Integer.toString(DEFAULT_HTTP_ACCEPTORS)));
    }

    /**
     * Get the default component properties
     *
     * @return
     */
    protected static Map<String, String> getDefaultProperties() {
        Map<String, String> result = new HashMap<String, String>(2);
        result.put(HTTP_PORT, Integer.toString(DEFAULT_HTTP_PORT));
        result.put(HTTP_SERVICES_LIST, Boolean
                .toString(DEFAULT_HTTP_SERVICES_LIST));
        result.put(HTTP_SERVICES_CONTEXT, DEFAULT_HTTP_SERVICES_CONTEXT);
        result.put(HTTP_SERVICES_MAPPING, DEFAULT_HTTP_SERVICES_MAPPING);
        result.put(HTTP_THREAD_POOL_SIZE_MAX, Integer
                .toString(DEFAULT_HTTP_THREAD_POOL_SIZE_MAX));
        result.put(HTTP_THREAD_POOL_SIZE_MIN, Integer
                .toString(DEFAULT_HTTP_THREAD_POOL_SIZE_MIN));
        result.put(HTTP_ACCEPTORS, Integer.toString(DEFAULT_HTTP_ACCEPTORS));

        return result;
    }

}
