/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap.listener.outgoing;

import static org.ow2.petals.binding.soap.Constants.ServiceUnit.ADD_ROOT;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.BASIC_AUTH_PASSWORD;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.BASIC_AUTH_USERNAME;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.CHUNKED_MODE;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.CLEANUP_TRANSPORT;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.PROXY_DOMAIN;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.PROXY_HOST;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.PROXY_PASSWORD;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.PROXY_PORT;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.PROXY_USER;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.SOAP_ACTION;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.SOAP_VERSION;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.apache.axiom.soap.SOAP11Constants;
import org.apache.axiom.soap.SOAP12Constants;
import org.apache.axis2.client.Options;
import org.apache.axis2.transport.http.HttpTransportProperties;
import org.apache.commons.httpclient.auth.AuthPolicy;
import org.ow2.petals.binding.soap.SoapComponentContext;
import org.ow2.petals.binding.soap.util.Marshaller;
import org.ow2.petals.component.framework.util.Extensions;
/**
 * @author chamerling - eBM WebSourcing
 *
 */
public abstract class AbstractJBIDispatcher implements JBIListenerDispatcher {

    public static final String SOAP_VERSION_11 = "11";

    public static final String SOAP_VERSION_12 = "12";

    /**
     * The soap component context
     */
    protected SoapComponentContext soapContext;

    /**
     * The logger
     */
    protected Logger logger;

    protected Marshaller marshaller = Marshaller.getInstance();

    /**
     * Creates a new instance of {@link AbstractJBIDispatcher}
     *
     * @param soapContext
     * @param logger
     */
    public AbstractJBIDispatcher(SoapComponentContext soapContext, Logger logger) {
        this.soapContext = soapContext;
        this.logger = logger;
    }
    
    /**
     *
     * @param extensions
     * @return
     */
    protected boolean retrieveAddRoot(Extensions extensions) {
        String result = extensions.getPetalsExtensions().getProperty(
                ADD_ROOT, "false");
        return Boolean.parseBoolean(result);
    }

    /**
     * Retrieve the default SOAP action from the extensions.
     *
     * @param extensions
     * @return
     */
    protected QName retrieveDefaultSOAPAction(Extensions extensions) {
        QName soapAction = null;
        String tmp = extensions.getPetalsExtensions().getProperty(
                SOAP_ACTION);

        if (tmp != null) {
            soapAction = QName.valueOf(tmp);
        }

        return soapAction;
    }

    /**
     * Retrieve the proxy settings from the extensions. The proxy-host value is
     * required. The other ones will be set to default values by Axis if they
     * have not been setted by the SU.
     *
     *
     * @param extensions
     *            must be not null. Contains <em>proxy-*</em> optional tags.
     *
     * @return the proxy settings if they are present in the extensions, null
     *         otherwise.
     * @see org.apache.axis2.transport.http.HttpTransportProperties.ProxyProperties
     *
     */
    protected HttpTransportProperties.ProxyProperties retrieveProxySettings(
            Extensions extensions) {

        HttpTransportProperties.ProxyProperties proxyProperties = null;

        if (extensions.getPetalsExtensions().getProperty(
                PROXY_HOST) != null) {
            // proxy host is required, if it is not set the proxy mode is not
            // activated

            proxyProperties = new HttpTransportProperties.ProxyProperties();

            String domain = extensions.getPetalsExtensions().getProperty(
                    PROXY_DOMAIN);
            String password = extensions.getPetalsExtensions().getProperty(
                    PROXY_PASSWORD);
            String proxyHost = extensions.getPetalsExtensions().getProperty(
                    PROXY_HOST);

            int proxyPort = -1;
            String tmp = extensions.getPetalsExtensions().getProperty(
                    PROXY_PORT);
            if (tmp != null) {
                try {
                    proxyPort = Integer.parseInt(tmp);
                } catch (NumberFormatException e) {
                }
            }

            String proxyUserName = extensions.getPetalsExtensions()
                    .getProperty(PROXY_USER);

            proxyProperties.setDomain(domain);
            proxyProperties.setPassWord(password);
            proxyProperties.setProxyName(proxyHost);
            proxyProperties.setProxyPort(proxyPort);
            proxyProperties.setUserName(proxyUserName);
        }

        return proxyProperties;
    }

    /**
     * Retrieve, in the jbi.xml descriptor, the SOAP version to use for
     * SoapEnvelopeNamespaceURI Search a <em>soap-version</em> tag. <br>
     * Legal values are <b>11</b> and <b>12</b>. 11 is used if no tag is
     * found.
     *
     * @param extensions
     *            must be non null. Contains a <em>soap-version</em> optional
     *            tag.
     * @return the Axis2 soapEnvelopeNamespaceURI constant.
     * @see org.apache.axiom.soap.SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI
     * @see org.apache.axiom.soap.SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI
     */
    protected String retrieveSOAPEnvelopeNamespaceURI(Extensions extensions) {
        String soapEnvelopeNamespaceURI = null;
        String soapVersion = extensions.getPetalsExtensions().getProperty(
                SOAP_VERSION);

        if (SOAP_VERSION_12.equals(soapVersion)) {
            soapEnvelopeNamespaceURI = SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI;
        } else {
            soapEnvelopeNamespaceURI = SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI;
        }
        return soapEnvelopeNamespaceURI;
    }
    
    /**
     * Get the cleanup-transport property value (true/false).
     * @see bug #308219
     * @param extensions
     * @return
     */
    protected boolean retrieveCleanupTransport(Extensions extensions) {
    	String clean = extensions.getPetalsExtensions().getProperty(
                CLEANUP_TRANSPORT, "true");
    	
    	return Boolean.valueOf(clean);
    }
    
    /**
     * Retrieve the chunked mode
     * 
     * @param extensions
     * @return
     */
    protected String retrieveChunkedMode(Extensions extensions) {
    	return extensions.getPetalsExtensions().getProperty(
                CHUNKED_MODE, "false");
    }

    /**
     * Getting the basic authentication parameters for the outgoing SOAP
     * request.
     *
     * @param extensions
     * @param options
     * @return
     */
    @SuppressWarnings("unchecked")
    protected HttpTransportProperties.Authenticator retrieveBasicAuthentication(
            Extensions extensions, Options options) {
        HttpTransportProperties.Authenticator basicAuthentication = null;
        String username = extensions.getPetalsExtensions().getProperty(
                BASIC_AUTH_USERNAME);
        String password = extensions.getPetalsExtensions().getProperty(
                BASIC_AUTH_PASSWORD);

        if (username != null) {
            if (password == null) {
                password = "";
            }

            basicAuthentication = new HttpTransportProperties.Authenticator();
            List schemes = new ArrayList();
            schemes.add(AuthPolicy.BASIC);
            basicAuthentication.setAuthSchemes(schemes);
            basicAuthentication.setUsername(username);
            basicAuthentication.setPassword(password);
            basicAuthentication.setPreemptiveAuthentication(true);
            options.setProperty(
                    org.apache.axis2.transport.http.HTTPConstants.AUTHENTICATE,
                    basicAuthentication);
        }

        return basicAuthentication;
    }
}
