/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIListener.java 154 27 sept. 06 alouis $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap.listener.outgoing;

import static org.ow2.petals.binding.soap.Constants.ServiceUnit.ADDRESS;

import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;

import org.ow2.petals.binding.soap.SoapComponent;
import org.ow2.petals.binding.soap.SoapComponentContext;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.component.framework.util.Extensions;

/**
 * Listener for incoming JBI messages. A SOAP message is created from the JBI
 * one.
 *
 * @author alouis - EBM Websourcing
 * @author Christophe Hamerling - EBM Websourcing
 *
 */
public class JBIListener extends AbstractJBIListener {

    /**
     * The logger
     */
    protected Logger logger;

    /**
     * Map of dispatchers
     */
    protected Map<String, JBIListenerDispatcher> dispatchers;

    @Override
    protected void init() {
        SoapComponentContext soapContext = ((SoapComponent) getComponent())
                .getSoapContext();
        this.logger = getLogger();
        JBIListenerDispatcher caller = new WSCaller(soapContext, this.logger, this.component);
        JBIListenerDispatcher notifier = new WSNotifier(soapContext,
                this.logger);
		// JBIListenerDispatcher rest = new RESTCaller(soapContext,
		// this.logger);

        // create dispatchers
        this.dispatchers = new HashMap<String, JBIListenerDispatcher>(3);
        this.dispatchers.put(caller.getDispatcherType(), caller);
        this.dispatchers.put(notifier.getDispatcherType(), notifier);
        // this.dispatchers.put(rest.getDispatcherType(), rest);
    }

    /**
     * Called by the JBI channel listener when a jbi message is accepted. The
     * call is dispatched like this :
     * <ul>
     * <li>If the extensions contains a topic-name property, the message will
     * be published in the topic</li>
     * <li>If there is no topic-name extension property, the JBI message is
     * sent to an external web service. The external web service address is the
     * one defined in the adress extension</li>
     * </ul>
     *
     * @param exchange
     * @param extensions
     */
    public void onJBIMessage(Exchange exchange) {
        if (MessageExchange.Role.PROVIDER.equals(exchange.getRole())) {
            String address = getExtensions().getValue(ADDRESS);

            logger
                    .fine("JBI message received on SOAP JBI listener for address : "
                            + address);

            this.dispatch(exchange, getExtensions(), address);
        }
    }

    /**
     * Dispatch the outgoing WS call.
     *
     * @param exchange
     * @param extensions
     * @param address
     */
    protected void dispatch(Exchange exchange,
            Extensions extensions, String address) {

        // Get the dispatcher key. The key is the start of the address
        // extension.
    	// FIXME : Get tanother way to retrive the dispatcher key
        String dispatchKey = address.substring(0, address.indexOf(":"));
        
        JBIListenerDispatcher dispatcher = dispatchers.get(dispatchKey);
        if (dispatcher == null) {
            // dispatcher is null, get the default one (hoping that it is
            // available)
            dispatcher = dispatchers
                    .get(JBIListenerDispatcher.DEFAULT_DISPATCHER);

            if (dispatcher == null) {
                try {
                    exchange.setFault(new SOAP11FaultServerException(
                            "No outgoing dispatcher found", address));
                } catch (URISyntaxException e) {
                    logger.log(Level.SEVERE, "Can't create SOAP11FaultServerException",
                            e);
                } catch (MessagingException e) {
                    logger.log(Level.SEVERE, "Can't return fault to consumer",
                            e);
                }
            }
        }

        if (dispatcher != null) {
            dispatcher.process(exchange, extensions);
        }
    }
}
