/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.binding.soap.listener.outgoing;

import static org.ow2.petals.binding.soap.Constants.ServiceUnit.ADDRESS;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Properties;
import java.util.logging.Logger;

import javax.xml.namespace.QName;
import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.ow2.petals.binding.soap.SoapComponentContext;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.component.framework.util.Extensions;

/**
 * A REST service caller. FIXME : To be completed...
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 3.0
 * 
 */
public class RESTCaller extends AbstractJBIDispatcher {

	/**
	 * GET, POST, PUT, DELETE
	 */
	public static final String REST_METHOD = "rest.method";

	public static final String DEFAULT_REST_METHOD = Constants.Configuration.HTTP_METHOD_GET;

	/**
	 * 
	 * @param soapContext
	 * @param logger
	 */
	public RESTCaller(SoapComponentContext soapContext, Logger logger) {
		super(soapContext, logger);
	}

	public String getDispatcherType() {
		return "rest";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.binding.soap.listener.outgoing.JBIListenerDispatcher#process(org.ow2.petals.component.framework.util.Exchange,
	 *      org.ow2.petals.component.framework.util.Extensions)
	 */
	public void process(Exchange exchange, Extensions extensions) {
		String address = extensions.getPetalsExtensions().getProperty(ADDRESS);

		this.logger.info("Calling REST service on " + address);

		String restMethod = getRESTMethod(exchange.getOperation(), extensions);
		String requestSuffix = buildRequest(exchange, extensions);
		String queryURL = buildQueryURL(address, requestSuffix);

		Options options = new Options();
		options.setTo(new EndpointReference(queryURL));
		options.setProperty(Constants.Configuration.ENABLE_REST,
				Constants.VALUE_TRUE);
		options.setProperty(Constants.Configuration.HTTP_METHOD, restMethod);

		OMElement result = null;
		try {
			ServiceClient sender = new ServiceClient();
			sender.setOptions(options);
			result = sender.sendReceive(getPayload(exchange));

			XMLStreamWriter writer = XMLOutputFactory.newInstance()
					.createXMLStreamWriter(System.out);
			result.serialize(writer);
			writer.flush();
		} catch (AxisFault e) {
			e.printStackTrace();
		} catch (XMLStreamException e) {
			e.printStackTrace();
		} catch (FactoryConfigurationError e) {
			e.printStackTrace();
		}
	}

	protected String buildRequest(Exchange exchange, Extensions extensions) {
		StringBuffer request = new StringBuffer();
		Properties props = extensions.getPetalsExtensions();
		String queryPattern = props.getProperty("query.pattern");

		int start = 0;
		while ((start = queryPattern.indexOf("${")) > -1) {
			int stop = queryPattern.indexOf("}");
			String key = queryPattern.substring(0, start);
			String placeholder = queryPattern.substring(start + 2, stop);
			queryPattern = queryPattern.substring(stop + 1, queryPattern
					.length());

			// get the value from the next exchange first, next from the
			// extensions, from the default values and
			// finally set it to empty string if no value has been found...
			String value = exchange.getProperty(placeholder).toString();
			if (value == null) {
				value = props.getProperty(placeholder).toString();
			}
			if (value == null) {
				value = props.getProperty(placeholder + ".default").toString();
			}
			if (value == null) {
				value = "";
			}

			request.append(key);
			try {
				request.append(URLEncoder.encode(value, "UTF-8"));
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		return request.toString();
	}

	/**
	 * Build the complete URL
	 * 
	 * @param address
	 * @param suffix
	 * @return
	 */
	protected String buildQueryURL(String address, String suffix) {
		StringBuffer sb = new StringBuffer(address);
		sb.append('/');
		sb.append(suffix);
		return sb.toString();
	}

	private OMElement getPayload(Exchange exchange) {
		OMFactory fac = OMAbstractFactory.getOMFactory();
		OMElement payload = fac.createOMElement("Dummy", null);
		return payload;
	}

	protected String getRESTMethod(QName jbiOperation, Extensions extensions) {
		String rest = null;

		if (jbiOperation != null) {
			rest = jbiOperation.getLocalPart();
		} else {
			rest = extensions.getPetalsExtensions().getProperty(REST_METHOD,
					DEFAULT_REST_METHOD);
		}
		return rest;
	}

	private static OMElement getPayload(String str) {
		OMFactory fac = OMAbstractFactory.getOMFactory();
		OMNamespace omNs = fac.createOMNamespace(
				"http://example1.org/example1", "example1");
		OMElement method = fac.createOMElement("echo", omNs);
		OMElement value = fac.createOMElement("Text", omNs);
		value.addChild(fac.createOMText(value, "Axis2 Echo String "));
		method.addChild(value);
		return method;
	}
}
