/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.binding.soap.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.axiom.om.OMElement;
import org.w3c.dom.Element;

/**
 * An utility class to manipulate AXIOM Elements.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @date Created on 15 janv. 08
 * @since 3.0
 * 
 */
public class AxiomUtils {

	/**
	 * 
	 */
	private static DocumentBuilderFactory documentBuilderFactory;

	/**
	 * Create a {@link ThreadLocal} in order to get only one builder for each
	 * thread instance.
	 */
	@SuppressWarnings("unchecked")
	private static ThreadLocal builderThreadLocal = new ThreadLocal() {
		protected Object initialValue() {
			try {
				DocumentBuilderFactory factory = getDocumentBuilderFactory();
				// factory.setValidating(false);
				factory.setNamespaceAware(true);
				DocumentBuilder builder = factory.newDocumentBuilder();
				return builder;
			} catch (ParserConfigurationException e) {
				throw new RuntimeException("Failed to create DocumentBuilder",
						e);
			}
		}
	};

	/**
	 * Taken from {@link org.apache.axis2.util.XMLUtils} Getting a new instance
	 * of DocumentBuilderfactory takes too much time...)
	 * 
	 * @param element
	 * @return
	 * @throws Exception
	 */
	public static Element toDOM(OMElement element) throws Exception {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		element.serialize(baos);
		ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
		return getDocumentBuilder().parse(bais).getDocumentElement();
	}

	/**
	 * Only get a new instance one time...
	 * 
	 * @return
	 */
	private static DocumentBuilderFactory getDocumentBuilderFactory() {
		if (documentBuilderFactory == null) {
			DocumentBuilderFactory f = DocumentBuilderFactory.newInstance();
			documentBuilderFactory = f;
		}
		return documentBuilderFactory;
	}

	/**
	 * Get the document builder. Since the {@link DocumentBuilder} is created in
	 * the {@link ThreadLocal}, we get only a {@link DocumentBuilder} instance
	 * by {@link Thread} instance. This is done since the
	 * <code>factory.getDocumentBuilder()</code> takes too much time and was
	 * previously called on each JBI message processing...
	 * 
	 * @return
	 */
	protected static DocumentBuilder getDocumentBuilder() {
		return (DocumentBuilder) builderThreadLocal.get();
	}
}
