/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap.util;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.wsdl.Operation;
import javax.wsdl.Part;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.Service;
import javax.xml.namespace.QName;

import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisOperation;
import org.apache.axis2.description.InOnlyAxisOperation;
import org.apache.axis2.description.InOutAxisOperation;
import org.apache.axis2.description.WSDL2Constants;

/**
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 *
 */
public class WsdlHelper {

    /**
     * Creates a new instance of {@link WsdlHelper}
     *
     */
    private WsdlHelper() {
    }

    /**
     *
     * @param definition
     * @param bodyName
     * @param serviceName
     * @param interfaceName
     * @param endpointName
     * @return
     */
    public static Operation getOperation(Definition definition, QName bodyName,
            QName serviceName, QName interfaceName, String endpointName) {
        Operation result = null;

        if (interfaceName != null) {
            PortType portType = definition.getPortType(interfaceName);
            if (portType != null) {
                result = getOperation(portType, bodyName);
            }
        } else if (definition.getService(serviceName) != null) {
            Service service = definition.getService(serviceName);
            if (endpointName != null) {
                Port port = service.getPort(endpointName);
                if (port != null) {
                    Binding binding = port.getBinding();
                    if (binding != null) {
                        PortType portType = binding.getPortType();
                        if (portType != null) {
                            result = getOperation(portType, bodyName);
                        }
                    }
                }
            } else if (service.getPorts().size() == 1) {
                Port port = (Port) service.getPorts().values().iterator()
                        .next();
                Binding binding = port.getBinding();
                if (binding != null) {
                    PortType portType = binding.getPortType();
                    if (portType != null) {
                        result = getOperation(portType, bodyName);
                    }
                }
            }
        } else if (definition.getPortTypes().size() == 1) {
            PortType portType = (PortType) definition.getPortTypes().values()
                    .iterator().next();
            result = getOperation(portType, bodyName);
        }
        return result;
    }

    @SuppressWarnings("unchecked")
    public static Operation getOperation(PortType portType, QName bodyName) {
        Operation result = null;
        boolean found = false;

        List<Operation> list = portType.getOperations();
        for (int i = 0; i < list.size(); i++) {
            Operation operation = (Operation) list.get(i);
            if (operation.getInput() != null
                    && operation.getInput().getMessage() != null) {
                Map parts = operation.getInput().getMessage().getParts();
                Iterator<Part> iter = parts.values().iterator();
                while (iter.hasNext() && !found) {
                    Part part = iter.next();
                    QName elementName = part.getElementName();
                    if (elementName != null && elementName.equals(bodyName)) {
                        result = operation;
                        found = true;
                    }
                }
            }
        }
        return result;
    }

    /**
     * Get the MEP from the operation. If no MEP is found, return the default
     * MEP (in opt out)
     *
     * @param operation
     * @return
     */
    public static String getMep(Operation operation) {
        String result = null;
        if (operation != null) {
            boolean output = operation.getOutput() != null
                    && operation.getOutput().getMessage() != null
                    && operation.getOutput().getMessage().getParts().size() > 0;

            boolean faults = operation.getFaults().size() > 0;

            if (output) {
                result = WSDL2Constants.MEP_URI_IN_OUT;
            } else if (faults) {
                result = WSDL2Constants.MEP_URI_ROBUST_IN_ONLY;
            } else {
                result = WSDL2Constants.MEP_URI_IN_ONLY;
            }
        }

        // set default MEP
        if (result == null) {
            result = WSDL2Constants.MEP_URI_IN_OPTIONAL_OUT;
        }
        return result;
    }

    /**
     * TODO
     *
     * @param msgContext
     * @return
     */
    public static AxisOperation getAxisOperation(MessageContext msgContext) {

        String mep = null;
        AxisOperation axisOperation = null;

        if (mep.equals(WSDL2Constants.MEP_URI_IN_ONLY)) {
            axisOperation = new InOnlyAxisOperation();
            axisOperation
                    .setMessageExchangePattern(WSDL2Constants.MEP_URI_IN_ONLY);
        } else if (mep.equals(WSDL2Constants.MEP_URI_IN_OUT)) {
            axisOperation = new InOutAxisOperation();
            axisOperation
                    .setMessageExchangePattern(WSDL2Constants.MEP_URI_IN_OUT);
        } else if (mep.equals(WSDL2Constants.MEP_URI_IN_OPTIONAL_OUT)) {
            axisOperation = new InOutAxisOperation();
            axisOperation
                    .setMessageExchangePattern(WSDL2Constants.MEP_URI_IN_OPTIONAL_OUT);
        } else if (mep.equals(WSDL2Constants.MEP_URI_ROBUST_IN_ONLY)) {
            axisOperation = new InOutAxisOperation();
            axisOperation
                    .setMessageExchangePattern(WSDL2Constants.MEP_URI_ROBUST_IN_ONLY);
        }
        return axisOperation;
    }
}
